@@ -192,6 +192,8 @@ module QueriesHelper
     value =
       if [:hours, :spent_hours, :total_spent_hours, :estimated_hours, :total_estimated_hours, :estimated_remaining_hours].include? column.name
         format_hours(value)
+      elsif column.is_a?(QueryCustomFieldColumn)
+        format_object(value, thousands_delimiter: column.custom_field.thousands_delimiter?)
       else
         format_object(value)
       end
@@ -225,7 +227,7 @@ module QueriesHelper
       end
       content =
         link_to(
-          sprite_icon(icon, column.caption),
+          icon ? sprite_icon(icon, column.caption) : column.caption,
           {:params => request.query_parameters.deep_merge(sort_param)},
           link_options
         )
@@ -480,8 +482,6 @@ module QueriesHelper
     url_params =
       if controller_name == 'issues'
         {:controller => 'issues', :action => 'index', :project_id => @project}
-      elsif controller_name == 'admin' && action_name == 'projects'
-        {:admin_projects => '1'}
       else
         {}
       end
@@ -510,7 +510,7 @@ module QueriesHelper
                               url_params.merge(:query_id => query),
                               :class => css,
                               :title => query.description,
-                              :data => { :disable_with => query.name }) +
+                              :data => { :disable_with => CGI.escapeHTML(query.name) }) +
                         clear_link.html_safe)
         end.join("\n").html_safe,
         :class => 'queries'
